.include "tn11def.inc"
;PortB
;Bit0: OFF \___/ for 3s after PortB3=0 for 10s
;Bit1: OK  \_/ 6s after PortB3=1 for ~100ms
;Bit2: ON  _/\_ for 3s 1 after PortB3=1 for 1s
;Bit3: +UB-car
;Bit4: GND
;Bit5: /RESET

.CSEG
.ORG 0
RES:    RJMP    Main
        RETI            ; IRQ0 handler
        RETI            ; Pin change handler
        RETI            ; Timer0 overflow handler
        RETI            ; Analog comparator
Main:   ;Enable Port change Interrupt for WakeUp
        CLI
        LDI     r16,0b00100000 ;PCIE
        OUT     GIMSK,r16
        SEI
        ;initialize Port B to all tristate inputs
        LDI     r16,0b00000000 ;Port B I/O Bits
        OUT     PORTB,r16
        LDI     r16,0b00000000 ;Port B direction Bits 0=Input
        OUT     DDRB,r16
        ;Wait for PB3 to become active for more than >0.3s
WaitON: LDI     r16,0b00110000 ;SE|SM: Enable Sleep, Power Down
        OUT     MCUCR,r16
        SBIS    PINB,3      ;Go to Power down if no external power present
        SLEEP
        CLR     r18         ;1
L2:     CLR     r19         ;1
L1:     SBIS    PINB,3      ;2
        RJMP    WaitON      ;1(/2)
        DEC     r19         ;1
        BRNE    L1          ;2(/1)
        ;Loop1: 6*256
        DEC     r18         ;1
        BRNE    L2          ;2(/1)
        ;(6*256+4)*256 = 394240 = ~0.4s
        ;Power for >0.6s, start GMini by pressing ON for >3s
        LDI     r16,0b00000100
        OUT     PORTB,r16   ;Set PullUp on Port B2
        OUT     DDRB,r16    ;Set Output on Port B2
        ;Wait>3s
        LDI     r17,35
        RCALL   DoWait
        LDI     r16,0b00000000
        OUT     DDRB,r16    ;Set Input on Port B2
        OUT     PORTB,r16   ;Set Port B2 tri-state
        ;Wait for >6s ~(3.5+1.5)
        LDI     r17,18
        RCALL   DoWait
        LDI     r16,0b00000010 ;Set Output on Port B1
        OUT     DDRB,r16
        ;Wait 200ms
        LDI     r17,2
        RCALL   DoWait
        LDI     r16,0b00000000 ;Set Port B1 tri-state
        OUT     DDRB,r16
        ;Now the GMini is on and continues the last played list

WaitOFF:;Wait for Power OFF
        LDI     r16,0b00110000 ;SE|SM: Enable Sleep, Power Down
        OUT     MCUCR,r16
        SBIC    PINB,3      ;Go to Power down while external power present
        SLEEP
        ;Wait 10s for Power to stay OFF
        LDI     r17,100      ;1
L5:     LDI     r18,66      ;1
L4:     LDI     r19,250     ;1
L3:     SBIC    PINB,3      ;2
        RJMP    WaitOFF     ;1(/2)
        DEC     r19         ;1
        BRNE    L3          ;2(/1)
        ;Loop3: 6*250
        DEC     r18         ;1
        BRNE    L4          ;2(/1)
        ;Loop4 (6*250+4)*66
        DEC     r17         ;1
        BRNE    L5          ;2(/1)
        ;LOOP5: ((6*250+4)*66+4)*100 = 10.08s

        ;NO Power for >10s, stop GMini by pressing OFF for >3s
        LDI     r16,0b00000001 ;Set Output on Port B0
        OUT     DDRB,r16
        ;Wait>3s
        LDI     r17,35
        RCALL   DoWait
        LDI     r16,0b00000000 ;Set Port B0 tri-state
        OUT     DDRB,r16
        ;Wait for Power ON
        RJMP    WaitON

DoWait: ;r17 in 100ms
L_2:    LDI     r18,100     ;1
L_1:    LDI     r19,249     ;1
L_0:    DEC     r19         ;1
        NOP                 ;1
        BRNE    L_0         ;2(/1)
        ;Loop0: 4*250 = 1000
        DEC     r18         ;1
        BRNE    L_1         ;2(/1)
        ;Loop1: (4*249+4)*100
        DEC     r17         ;1
        BRNE    L_2         ;2(/1)
        ;Loop2: ((4*249+4)*250+4)*r17 = 100.004ms * r17
        RET
